"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.processMetadata = void 0;
const crypto_1 = __importDefault(require("crypto"));
const metascraper_1 = __importDefault(require("metascraper"));
const metascraper_description_1 = __importDefault(require("metascraper-description"));
const metascraper_image_1 = __importDefault(require("metascraper-image"));
const metascraper_title_1 = __importDefault(require("metascraper-title"));
const metascraper_url_1 = __importDefault(require("metascraper-url"));
const constants_1 = require("../constants");
const scraper = (0, metascraper_1.default)([
    (0, metascraper_url_1.default)(),
    (0, metascraper_title_1.default)(),
    (0, metascraper_description_1.default)(),
    (0, metascraper_image_1.default)(),
]);
const processMetadata = async (blocks, actions, createNodeId, reporter, cache) => {
    const { createNode } = actions;
    for (const block of blocks) {
        if (block.type === "text" ||
            block.type === "paragraph" ||
            block.type === "bulleted_list_item" ||
            block.type === "numbered_list_item" ||
            block.type === "quote" ||
            block.type === "callout") {
            const richText = block[block.type]?.rich_text || [];
            for (const text of richText) {
                const href = text.href;
                if (href) {
                    try {
                        reporter.info(`[INFO] Processing metadata for URL: ${href}`);
                        const cacheKey = `metadata-${crypto_1.default
                            .createHash("md5")
                            .update(href)
                            .digest("hex")}`;
                        const cachedMetadata = await cache.get(cacheKey);
                        if (cachedMetadata) {
                            reporter.info(`[INFO] Using cached metadata for URL: ${href}`);
                            text.href = cachedMetadata.nodeId;
                            continue;
                        }
                        reporter.info(`[INFO] Fetching metadata for URL: ${href}`);
                        const response = await fetch(href, {
                            headers: { "User-Agent": "Mozilla/5.0" },
                        });
                        const html = await response.text();
                        const metadata = await scraper({ html, url: href });
                        const nodeId = createNodeId(`${crypto_1.default.createHash("md5").update(href).digest("hex")}-metadata`);
                        const metadataNode = {
                            id: nodeId,
                            parent: null,
                            children: [],
                            internal: {
                                type: constants_1.NODE_TYPE.Metadata,
                                contentDigest: crypto_1.default
                                    .createHash("md5")
                                    .update(JSON.stringify(metadata))
                                    .digest("hex"),
                            },
                            title: metadata.title || "",
                            description: metadata.description || "",
                            image: metadata.image || "",
                            url: metadata.url || href,
                        };
                        createNode(metadataNode);
                        reporter.info(`[SUCCESS] Created metadata node for URL: ${href}`);
                        await cache.set(cacheKey, { ...metadata, nodeId });
                        text.href = nodeId;
                    }
                    catch (error) {
                        if (error instanceof Error) {
                            reporter.error(`[ERROR] Failed to fetch metadata for URL: ${href}`, error);
                        }
                        else {
                            console.error("Unknown error fetching metadata:", error);
                        }
                    }
                }
            }
        }
    }
    return blocks;
};
exports.processMetadata = processMetadata;
