"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.processor = void 0;
const gatsby_source_filesystem_1 = require("gatsby-source-filesystem");
const metadataProcessor_1 = require("./metadataProcessor");
const tableOfContent_1 = require("./tableOfContent");
const processor = async (blocks, actions, getCache, createNodeId, reporter, cache) => {
    const { thumbnail, tableOfContents, updatedBlocks, rawText } = await processBlocksForContent(blocks, actions, getCache, createNodeId, reporter, cache);
    await (0, metadataProcessor_1.processMetadata)(blocks, actions, createNodeId, reporter, cache);
    return [thumbnail, tableOfContents, updatedBlocks, rawText];
};
exports.processor = processor;
const processBlocksForContent = async (blocks, actions, getCache, createNodeId, reporter, cache) => {
    const tableOfContents = [];
    let thumbnail = null;
    let rawText = "";
    const updatedBlocks = (await Promise.all(blocks.map(async (block) => {
        await (0, tableOfContent_1.processTableOfContents)(block, tableOfContents);
        const plainText = extractPlainText(block);
        if (plainText) {
            rawText += plainText + " ";
        }
        if (isImageBlock(block)) {
            const updatedBlock = await processImageBlock(block, actions, getCache, createNodeId, reporter, cache);
            if (!thumbnail && updatedBlock?.image?.fileId) {
                thumbnail = updatedBlock.image.fileId;
            }
            return updatedBlock;
        }
        return block;
    }))).filter((block) => block !== null);
    return { thumbnail, tableOfContents, updatedBlocks, rawText };
};
const isTextContentBlock = (block) => {
    return [
        "paragraph",
        "heading_1",
        "heading_2",
        "heading_3",
        "quote",
        "bulleted_list_item",
        "numbered_list_item",
        "callout",
        "code",
    ].includes(block.type);
};
const extractPlainText = (block) => {
    if (isTextContentBlock(block)) {
        const richTextArray = block[block.type]?.rich_text || [];
        return richTextArray
            .map((text) => block.type === "code" // code의 \n 제거
            ? text.plain_text.replace(/\\n/g, "")
            : text.plain_text)
            .join(" ");
    }
    return null;
};
const isImageBlock = (block) => {
    return block.type === "image" && "image" in block;
};
const processImageBlock = async (block, actions, getCache, createNodeId, reporter, cache) => {
    const { createNode } = actions;
    if (block.type === "image" && "image" in block) {
        const imageSourceType = block.image.type;
        const imageUrl = imageSourceType === `external`
            ? block.image.external?.url
            : block.image?.file?.url;
        if (!imageUrl)
            return null;
        const cacheKey = `${imageUrl}-post-image`;
        const cachedFileNodeId = await cache.get(cacheKey);
        if (cachedFileNodeId) {
            reporter.info(`[CACHE HIT] Image already processed: ${imageUrl}`);
            const updatedBlock = {
                ...block,
                image: {
                    fileId: cachedFileNodeId,
                    caption: block.image.caption,
                },
            };
            return updatedBlock;
        }
        try {
            const fileNode = await (0, gatsby_source_filesystem_1.createRemoteFileNode)({
                url: imageUrl,
                parentNodeId: block.id,
                getCache,
                createNode,
                createNodeId,
            });
            if (fileNode) {
                const updatedBlock = {
                    ...block,
                    image: {
                        fileId: fileNode.id,
                        caption: block.image.caption,
                    },
                };
                reporter.info(`[SUCCESS] Image processed: ${fileNode.id}`);
                await cache.set(cacheKey, fileNode.id);
                return updatedBlock;
            }
        }
        catch (error) {
            reporter.warn(`[WARNING] Failed to download image: ${imageUrl}`);
        }
    }
    return null;
};
